@@ -1,6 +1,6 @@
 package Mason::Tidy::t::CLI;
 BEGIN {
-  $Mason::Tidy::t::CLI::VERSION = '2.56';
+  $Mason::Tidy::t::CLI::VERSION = '2.57';
 }
 use Capture::Tiny qw(capture capture_merged);
 use File::Slurp;
@@ -31,14 +31,14 @@ sub test_cli : Tests {
     };
 
     $cli->( "-r", "$tempdir/comp1.mc", "$tempdir/comp2.mc", @std_argv );
-    is( $out,                           "",              "out empty" );
-    is( read_file("$tempdir/comp1.mc"), "<% 2 + 2 %>\n", "comp1" );
-    is( read_file("$tempdir/comp2.mc"), "<% 4 + 4 %>\n", "comp2" );
+    is( $out,                           "$tempdir/comp1.mc\n$tempdir/comp2.mc\n", "out empty" );
+    is( read_file("$tempdir/comp1.mc"), "<% 2 + 2 %>",                            "comp1" );
+    is( read_file("$tempdir/comp2.mc"), "<% 4 + 4 %>",                            "comp2" );
 
     write_file( "$tempdir/comp1.mc", "<%2+2%>" );
     $cli->( "$tempdir/comp1.mc", @std_argv );
-    is( $out,                           "<% 2 + 2 %>\n", "single file - out" );
-    is( read_file("$tempdir/comp1.mc"), "<%2+2%>",       "comp1" );
+    is( $out,                           "<% 2 + 2 %>", "single file - out" );
+    is( read_file("$tempdir/comp1.mc"), "<%2+2%>",     "comp1" );
 
     $cli->( "$tempdir/comp3.mc", @std_argv );
     is( $out, "% if (foo) {\n%     bar\n% }\n", "no options" );
@@ -71,6 +71,10 @@ sub test_usage : Tests {
     return "author only" unless ( $ENV{AUTHOR_TESTING} );
     $out = capture_merged { system( $^X, "bin/masontidy", "-h" ) };
     like( $out, qr/Usage: masontidy/ );
+
+    $out = capture_merged { system( $^X, "bin/masontidy", "--version" ) };
+    like( $out, qr/masontidy .* on perl/ );
+
 }
 
 1;
